import * as THREE from "../build/three.module.js"
import Stats from "./jsm/libs/stats.module.js"

class App {
    constructor() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        renderer.outputEncoding = THREE.sRGBEncoding;
        renderer.setPixelRatio(window.devicePixelRatio);
        //renderer.shadowMap.enabled =  true;
        domWebGL.appendChild(renderer.domElement);
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0x101010);
        this._scene = scene;
        
        this._setupCamera();
        //this._setupLight();
        this._setupModel();
        //this._setupInteractions();
        this._setupGUI();
        //this._setupControls();
        //this._setupHelper();
        
        window.onresize = this.resize.bind(this);

        this.resize();
        this.render()
    }

    _setupInteractions() {
        //.
    }

    _setupPostprocessing() {
        //.
    }

    _setupModel() {
        const vertexCount = 200 * 3;
        const geometry = new THREE.BufferGeometry();
        const positions = [];
        const colors = [];

        for(let i=0; i<vertexCount; i++) {
            positions.push(Math.random() - 0.5);
            positions.push(Math.random() - 0.5);
            positions.push(Math.random() - 0.5);

            colors.push(Math.random() * 255);
            colors.push(Math.random() * 255);
            colors.push(Math.random() * 255);
            colors.push(Math.random() * 255);
        }

        const positionAttribute = new THREE.Float32BufferAttribute(positions, 3);
        const colorAttribute = new THREE.Uint8BufferAttribute(colors, 4);

        colorAttribute.normalized = true; // 버퍼의 값이 쉐이더 내부에서 0-1 사이의 값으로 정규화됨

        geometry.setAttribute("position", positionAttribute);
        geometry.setAttribute("color", colorAttribute);

        const vertexShader = `
            precision mediump float;
            precision mediump int;

            uniform mat4 modelViewMatrix;
			uniform mat4 projectionMatrix;

            attribute vec3 position;
            attribute vec4 color;

            varying vec3 vPosition;
            varying vec4 vColor;

            void main() {
                vPosition = position;
                vColor = color;

                gl_Position = projectionMatrix * modelViewMatrix * vec4(position, 1.0);
            }
        `;

        const fragmentShader = `
            precision mediump float;
            precision mediump int;

            uniform float time;

            varying vec3 vPosition;
            varying vec4 vColor;

            void main() {
                vec4 color = vec4(vColor);
                
                color.r += sin(vPosition.x * 7.0 + time) * 0.5;
                color.g += cos(vPosition.y * 5.0 + time) * 0.5;
                color.b += sin(vPosition.z * 3.0 + time) * 0.5;

                gl_FragColor = color;
            }
        `;

        const material = new THREE.RawShaderMaterial({
            uniforms: {
                time: { value: 1.0 }
            },
            vertexShader: vertexShader,
            fragmentShader: fragmentShader,
            side: THREE.DoubleSide,
            transparent: true
        });

        const mesh = new THREE.Mesh(geometry, material);
        this._scene.add(mesh);
    }

    _setupHelper() {
        //.
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);
        this._stats = stats;
    }

    _setupCamera() {    
        const camera = new THREE.PerspectiveCamera(50, window.innerWidth / window.innerHeight, 1, 10);
        camera.position.z = 2;

        this._camera = camera;
    }

    _setupControls() {
        //.
    }

    _setupLight() {
        //.
    }

    update() {
        const time = performance.now();
        
        const object = this._scene.children[0];
        object.rotation.y = time * 0.0005;
        object.material.uniforms.time.value = time * 0.01;

        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        this._camera.aspect = window.innerWidth / window.innerHeight;
        this._camera.updateProjectionMatrix();
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    new App();
}